#!/usr/bin/env python
__author__    = 'Hoby Rakotoarivelo'
__license__   = 'GPL v3'
__version__   = '1.0'
__copyright__ = 'Copyright 2016, The trinity project'

import os

def generate_gnuplot_script(param):

    testcase = param.name
    nb_cores = int(param.cores)

    data  = 'benchs/'+testcase    # data path
    index = 4
    stats = [''] * 4
    t_seq = [''] * 4
    color = [''] * 4

    for i in range(0,4):
        stats[i] = 'results/'+data+'_'+str(i+1)+'.dat'
        assert(os.path.exists(stats[i]))
        with open(stats[i],'r') as f:
            line = f.readline().split()
            t_seq[i] = line[index]
            #print('>> t_seq['+str(i+1)+'] = '+str(t_seq[i]))

        assert(int(t_seq[i]) > 0)

    color[0] = '#4682B4'
    color[1] = '#778899'
    color[2] = '#000050'
    color[3] = '#800080'

    s  = '# ---------------------------------\n'
    s += '# '+testcase+'.gnuplot\n'
    s += '# generated by trigen '+time.strftime('%c')+'\n'
    s += '# (c) 2016 H. Rakotoarivelo\n'
    s += '# ---------------------------------\n'
    s += 'reset\n'
    s += 'set terminal postscript eps enhanced color 14 size 6.5cm,6.5cm\n'
    s += 'set output "../'+testcase+'_runtime.eps"\n'
    s += 'set size ratio 1\n'
    s += 'set xlabel "cores"\n'
    s += 'set ylabel "elapsed time (s)"\n'
    s += 'set format y "%.0f"\n'
    s += 'set logscale x 2\n'
    s += 'set grid\n'
    s += 'plot "'+testcase+'_1.dat" using 1:($5/1e3) with linespoint title "refinement"  '
    s += 'lc rgb "'+color[0]+'" pt 5,\\\n'
    s += '     "'+testcase+'_2.dat" using 1:($5/1e3) with linespoint title "contraction" '
    s += 'lc rgb "'+color[1]+'" pt 3,\\\n'
    s += '     "'+testcase+'_3.dat" using 1:($5/1e3) with linespoint title "swapping"    '
    s += 'lc rgb "'+color[2]+'" pt 3,\\\n'
    s += '     "'+testcase+'_4.dat" using 1:($5/(3*1e3)) with linespoint title "smoothing (x3)"   '
    s += 'lc rgb "'+color[3]+'" pt 1\n'
    s += '# ---------------------------------\n'
    s += '\n'
    s += 'reset\n'
    s += 'set terminal postscript eps enhanced color 14 size 6.5cm,6.5cm\n'
    s += 'set output "../'+testcase+'_speedup.eps"\n'
    s += 'set size ratio 1\n'
    s += 'unset logscale x\n'
    s += 'set key left\n'
    s += 'set ylabel "speedup"\n'
    s += 'set xlabel "cores"\n'
    s += 'set yrange [1:'+str(nb_cores)+']\n'
    s += 'set ytics '+str(int(nb_cores/8))+'\n'
    s += '\n'
    s += 't1 = '+t_seq[0]+'\n'
    s += 't2 = '+t_seq[1]+'\n'
    s += 't3 = '+t_seq[2]+'\n'
    s += 't4 = '+t_seq[3]+'\n'
    s += '\n'
    s += 'set style data histogram\n'
    s += 'set style fill solid border\n'
    s += 'set style histogram clustered\n'
    s += 'set grid\n'
    s += '# ---------------------------------\n'
    s += 'plot "'+testcase+'_1.dat" using (t1/$5):xticlabels(1) title "refinement"  '
    s += ' lc rgb "'+color[0]+'",\\\n'
    s += '     "'+testcase+'_2.dat" using (t2/$5):xticlabels(1) title "contraction" '
    s += ' lc rgb "'+color[1]+'",\\\n'
    s += '     "'+testcase+'_3.dat" using (t3/$5):xticlabels(1) title "swapping"    '
    s += ' lc rgb "'+color[2]+'",\\\n'
    s += '     "'+testcase+'_4.dat" using (t4/$5):xticlabels(1) title "smoothing"   '
    s += ' lc rgb "'+color[3]+'"\n'
    s += '# ---------------------------------'

    # a) store to file
    script = testcase+'.gnuplot'
    path = 'results/benchs/'+script
    print('>> '+path)
    with open(path,'w') as output:
        output.write(s)

    # b) execute gnuplot script
    print('>> cd results/benchs')
    os.chdir('results/benchs')
    print('>> gnuplot '+script)
    os.system('gnuplpot '+script)
    #subprocess.call(['gnuplot', script])

    print('>> results/'+testcase+'_runtime.eps')
    print('>> results/'+testcase+'_speedup.eps')
